@@ -171,12 +171,22 @@ sub from_sets {
 }
 
 sub start_set {
+    if ( exists $_[0]->{set}{method} &&
+         $_[0]->{set}{method} eq 'until' )
+    {
+        return bless { set => $_[0]->{set}{parent}[0] }, 'DateTime::Set';
+    }
     my $return = DateTime::Set->empty_set;
     $return->{set} = $_[0]->{set}->start_set;
     $return;
 }
 
 sub end_set {
+    if ( exists $_[0]->{set}{method} &&
+         $_[0]->{set}{method} eq 'until' )
+    {
+        return bless { set => $_[0]->{set}{parent}[1] }, 'DateTime::Set';
+    }
     my $return = DateTime::Set->empty_set;
     $return->{set} = $_[0]->{set}->end_set;
     $return;
@@ -377,8 +387,17 @@ sub intersected_spans {
 
 sub intersects {
     my ($set1, $set2) = ( shift, shift );
+    
+    unless ( $set2->can( 'union' ) )
+    {
+        for ( $set2, @_ )
+        {
+            return 1 if $set1->contains( $_ );
+        }
+        return 0;
+    }
+    
     my $class = ref($set1);
-    my $tmp = $class->empty_set();
     $set2 = $set2->as_spanset
         if $set2->can( 'as_spanset' );
     $set2 = $set2->as_set
@@ -390,8 +409,32 @@ sub intersects {
 
 sub contains {
     my ($set1, $set2) = ( shift, shift );
+    
+    unless ( $set2->can( 'union' ) )
+    {
+        if ( exists $set1->{set}{method} &&
+             $set1->{set}{method} eq 'until' )
+        {
+            my $start_set = $set1->start_set;
+            my $end_set =   $set1->end_set;
+
+            for ( $set2, @_ )
+            {
+                my $start = $start_set->next( $set2 );
+                my $end =   $end_set->next( $set2 );
+
+                goto ABORT unless defined $start && defined $end;
+            
+                return 0 if $start < $end;
+            }
+            return 1;
+
+            ABORT: ;
+            # don't know 
+        }
+    }
+    
     my $class = ref($set1);
-    my $tmp = $class->empty_set();
     $set2 = $set2->as_spanset
         if $set2->can( 'as_spanset' );
     $set2 = $set2->as_set
@@ -453,7 +496,10 @@ sub span {
 sub duration { 
     my $dur; 
 
-    eval { $dur = $_[0]->{set}->size };
+    eval { 
+        local $SIG{__DIE__};   # don't want to trap this (rt ticket 5434)
+        $dur = $_[0]->{set}->size 
+    };
     $@ = undef;  # clear the eval() error message
 
     return $dur if defined $dur && ref( $dur );
@@ -637,16 +683,32 @@ returns more than one span.
 
 Returns a list of C<DateTime::Span> objects.
 
-  my @dt = $set->as_list( span => $span );
+  my @dt_span = $set->as_list( span => $span );
 
 Just as with the C<iterator()> method, the C<as_list()> method can be
 limited by a span.
 
-If a set is specified as a recurrence and has no
-fixed begin and end datetimes, then C<as_list> will return C<undef>
-unless you limit it with a span.  Please note that this is explicitly
-not an empty list, since an empty list is a valid return value for
-empty sets!
+Applying C<as_list()> to a large recurring spanset is a very expensive operation, 
+both in CPU time and in the memory used.
+
+For this reason, when C<as_list()> operates on large recurrence sets, it will 
+return at most approximately 200 spans. For larger sets, and for I<infinite> 
+sets, C<as_list()> will return C<undef>.
+
+Please note that this is explicitly not an empty list, since an empty list is a 
+valid return value for empty sets!
+
+If you I<really> need to extract spans from a large set, you can:
+
+- limit the set with a shorter span:
+
+    my @short_list = $large_set->as_list( span => $short_span );
+
+- use an iterator:
+
+    my @large_list;
+    my $iter = $large_set->iterator;
+    push @large_list, $dt while $dt = $iter->next;
 
 =item * union
 
